(function(){
  if(window.SimpleSlideshow) return; //Singleton

  var slideshows = [];

  function SimpleSlideshow(selector){
    var container = selector;
    if(!container) return null;

    if (typeof selector === 'string' || selector instanceof String) container = document.querySelector(selector);
    if(!container.classList.contains('SimpleSlideshow')) container = container.querySelector('.SimpleSlideshow');
    if(!container) return null;
    if(slideshows.indexOf(container)>=0){ return slideshows[container]; }

    slideshows.push(container);
    var slideshowId = slideshows.length-1;

    var _this = this;

    this.slides = container.children;
    this.slideIndex = 0;
    this.isRunning = false;
    this.timer = null;
    this.slideTime = 2000;
    this.fadeTime = 1000;
    this.timeFrame = 0;
    this.lastTimer = 0;

    container.style.position = 'relative';
    container.style.overflow = 'hidden';
    for(var i=0;i<_this.slides.length;i++){
      var slide = _this.slides[i];
      if(i==0){
        slide.style.position = 'relative';
      }
      else {
        slide.style.position = 'absolute';
        slide.style.width = '100%';
        slide.style.boxSizing = 'border-box';
      }
      slide.style.top = '0px';
      slide.style.left = '0px';
    }

    this.start = function(){
      if(_this.isRunning) return;
      if(_this.slides.length <= 1) return;
      _this.isRunning = true;
      _this.timeFrame = 0;
      _this.lastTimer = new Date().getTime();
      _this.animate();
    }
  
    this.stop = function(){
      if(!_this.isRunning) return;
      _this.isRunning = false;
      if(_this.timer){
        clearTimeout(_this.timer);
        _this.timer = null;
      }
      //Stop animation
      _this.slides[_this.slideIndex].style.opacity = 1;
    }
  
    this.animate = function(){
      if(!_this.isRunning) return;
      var curTime = new Date().getTime();
      _this.timeFrame += (curTime - this.lastTimer);

      if(_this.timeFrame < _this.slideTime){
        //Do nothing
      }
      else if(_this.timeFrame < _this.slideTime + _this.fadeTime){
        //Animate topmost slide
        var opacity = 1-(_this.timeFrame - _this.slideTime) / _this.fadeTime;
        if(opacity < 0) opacity = 0;
        if(opacity > 1) opacity = 1;
        _this.slides[_this.slideIndex].style.opacity = opacity;
      }
      else {
        //Transition
        var nextIndex = _this.slideIndex + 1;
        _this.moveTo(nextIndex);
      }

      this.lastTimer = curTime;
      this.timer = setTimeout(function(){ _this.animate() }, 50);
    }

    this.moveTo = function(nextIndex){
      if(nextIndex >= _this.slides.length) nextIndex = 0;
      else if(nextIndex < 0) nextIndex = _this.slides.length - 1;

      var oldIndex = _this.slideIndex;
      _this.slideIndex = nextIndex;
      
      //Set slide z-indexes
      for(var i=0;i<_this.slides.length;i++){
        _this.slides[i].style.zIndex = (_this.slideIndex+_this.slides.length-i-1) % _this.slides.length;
      }
      //Set initial styles
      _this.slides[oldIndex].style.opacity = 1;
      //Set variables
      _this.timeFrame = 0;
    }

    this.moveTo(0);
    this.start();
    return this;
  }

  //Initialize for all SimpleSlideshow elements
  setTimeout(function(){
    var existingSlideshows = document.querySelectorAll('.SimpleSlideshow');
    for(var i=0;i<existingSlideshows.length;i++){
      new SimpleSlideshow(existingSlideshows[i]);
    }
  }, 1);

  window.SimpleSlideshow = function(selector){
    return new SimpleSlideshow(selector);
  };
})();